module Fluent
    class IlmHbaseOutput < Fluent::BufferedOutput
    Fluent::Plugin.register_output('ilm_hbase', self)

    helpers :formatter
    def initialize
      super
      require 'massive_record'
    end

    # Format dates with ISO 8606 by default
    #     # http://www.w3.org/TR/NOTE-datetime
    config_param :time_format, :string, :default => '%Y-%m-%dT%H:%M:%S.%L%:z'

    include SetTagKeyMixin
    config_set_default :include_tag_key, false
    config_set_default :tag_key, nil

    include SetTimeKeyMixin
    config_set_default :include_time_key, false
    config_set_default :time_key, nil

    config_param :use_format, :bool, :default=>false
    config_param :include_custom_row_key, :bool, :default=>false
    config_param :custom_key_list, :string, :default => nil
    config_param :tag_column_name, :string, :default => nil
    config_param :time_column_name, :string, :default => nil
    config_param :fields_to_columns_mapping, :string
    config_param :hbase_host, :string, :default => 'localhost'
    config_param :hbase_port, :integer, :default => 9090
    config_param :hbase_table, :string

    DEFAULT_FORMAT_TYPE = "out_file"

    config_section :format do
      config_set_default :@type, DEFAULT_FORMAT_TYPE
    end

    def configure(conf)
      super
      @fields_to_columns = @fields_to_columns_mapping.split(",").map { |src_to_dst|
        src_to_dst.split("=>")
      }
      @mapping = Hash[*@fields_to_columns.flatten]
      @rowchangeflag = @include_custom_row_key

      if(@rowchangeflag == true)
      @rowkeys = @custom_key_list.split(",")
      end
      
      @customrow = SecureRandom.uuid
      @formatter = formatter_create
    end

    def start
      super

      @conn = MassiveRecord::Wrapper::Connection.new(:host => @hbase_host, :port => @hbase_port)
      @table = MassiveRecord::Wrapper::Table.new(@conn, @hbase_table.intern)

      unless @table.exists?
        columns = ([@tag_column_name, @time_column_name] + @mapping.values).reject(&:nil?)
        column_families = columns.map {|column_family_with_column|
          column_family, column = column_family_with_column.split(":")

          if column.nil? or column_family.nil?
            raise <<MESSAGE
Unexpected format for column name: #{column_family_with_column}
Each destination column in the 'record_to_columns_mapping' option
must be specified in the format of \"column_family:column\".
Are you sure you included ':' in column names?
MESSAGE
          end

          column_family.intern
        }
        column_families.uniq!

        @table.create_column_families(column_families)
        @table.save
      end
    end

    def format(tag, time, record)
      row_values = {}
      new_record = (use_format==true)? @formatter.format(tag, time, record):record
      row_values[@tag_column_name] = tag unless @tag_column_name.nil?
      row_values[@time_column_name] = time unless @time_column_name.nil?

	log.warn("#{record.values}")
	log.warn("#{new_record.values}")
      @fields_to_columns.each {|field,column|

        next if field.nil? or column.nil?

        components = field.split(".")
        value = new_record
        for c in components
          value = value[c]

          break if value.nil?
        end

        row_values[column] = value
      }
	log.warn("test point 1")
      row_values.to_msgpack
    end

    def write(chunk)
        row_key = ""
	row_attrs = Array.new
	if @rowchangeflag
        chunk.msgpack_each {|row_values|
        
	row_attrs << row_values.each do |column_family_and_column, value| 
		if  @rowkeys.include? column_family_and_column
		  row_key << value
		  row_key << "-"
		end
	end
   
	}
	@customrow = SecureRandom.uuid + ":" + row_key[0,row_key.length-1]
	else
	@customrow = SecureRandom.uuid
	end
log.warn("test point 2")
      chunk.msgpack_each {|row_values|
        event = {}

        row_values.each {|column_family_and_column, value|
          column_family, column = column_family_and_column.split(":")

          (event[column_family.intern] ||= {}).update({column => value})
        }
log.warn("test point 3")
        row = MassiveRecord::Wrapper::Row.new
        row.id = @customrow
        row.values = event
        row.table = @table
        row.save
      }
    end
    end
  end
